#include <stdio.h>
#include <stdlib.h>

#include <libusb-1.0/libusb.h>

static libusb_context *context = NULL;
static libusb_device_handle *handle = NULL;

union axis{
    struct hx{
        unsigned char h:4;
        unsigned char u:2;
        unsigned char f:2;
    } x;

    struct hy{
        unsigned char h:4;
        unsigned char id:4;
    } y;

    char c;
};

struct point {
    union axis xh;
    unsigned char xl;
    union axis yh;
    unsigned char yl;

    unsigned char weight;
    unsigned char misc;
};

struct touch {
    unsigned char unused[2];
    unsigned char count;
    struct point p[2];
};

int main(int argc, char *argv[])
{
    unsigned char cmd[128] = {0};
    struct touch *t = (struct touch *)cmd;
    int used = 0;

    libusb_init(&context);
    handle = libusb_open_device_with_vid_pid(context, 0xc872, 0x1004);
    if (handle == NULL) {
        printf("no device.\n");
        return -1;
    }

    libusb_claim_interface(handle, 0);
    libusb_set_interface_alt_setting(handle, 0, 0);

    while (1) {
        int x1, x2, y1, y2, t1, t2;
        char map[][6] = {"PRESS", "HOVER", "DRAG"};
        argc = libusb_interrupt_transfer(handle, 0x81, cmd, 128, &used, -1);

        x1 = (((int)t->p[0].xh.x.h) << 8) + t->p[0].xl;
        y1 = (((int)t->p[0].yh.y.h) << 8) + t->p[0].yl;
        t1 = t->p[0].xh.x.f;
        x2 = (((int)t->p[1].xh.x.h) << 8) + t->p[1].xl;
        y2 = (((int)t->p[1].yh.y.h) << 8) + t->p[1].yl;
        t2 = t->p[1].xh.x.f;
        
        if (x2 == 4095 && y2 == 4095)
            printf("%d:%s (%d, %d)\n", t->p[0].yh.y.id, map[t1], x1, y1);
        else
            printf("%d:%s (%d, %d), %d:%s (%d, %d)\n", 
                   t->p[0].yh.y.id, map[t1], x1, y1, t->p[1].yh.y.id, map[t2], x2, y2);
    }

    libusb_release_interface(handle, 0);
    libusb_close(handle);
    libusb_exit(context);
    return 0;
}
