#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <memory.h>
#include <libusb-1.0/libusb.h>

static libusb_context *context = NULL;
static libusb_device_handle *handle = NULL;

#define FRAMESIZE  768000   // 800x480x16bit

int main(int argc, char *argv[])
{
    if (argc != 2) {
        printf("send_frame [bmp16 data]\n");
        return 0;
    }
    
    // open usb port.
    libusb_init(&context);
    handle = libusb_open_device_with_vid_pid(context, 0xc872, 0x1004);
    if (handle == NULL) {
        printf("no device.\n");
        return -1;
    }

    libusb_claim_interface(handle, 0);
    libusb_set_interface_alt_setting(handle, 0, 0);

    // quit sleep mode, or it will show white screen as default.
    uint8_t cmd1[] = {0x00, 0x29};
    libusb_control_transfer(handle, 0x40, 0xb0, 0, 0, cmd1, 2, 100);

    // read data to buffer and write to screen.
    uint8_t *frame = malloc(FRAMESIZE);
    FILE *fp = fopen(argv[1], "rb");
    
    if (fp == NULL || FRAMESIZE != fread(frame, 1, FRAMESIZE, fp)) {
        printf("not enough data.\n");
        return -1;
    }
    fclose(fp);
    
    // write start frame command 0x2c, size 768000.
    uint8_t cmd2[] = {0x00, 0x2c, 0x00, 0xb8, 0x0b, 0x00};
    libusb_control_transfer(handle, 0x40, 0xb0, 0, 0, cmd2, 6, 100);
    
    // send data to screen.
    libusb_bulk_transfer(handle, 0x02, frame, FRAMESIZE, NULL, 100);
    
    free(frame);

    // close usb port.
    libusb_release_interface(handle, 0);
    libusb_close(handle);
    libusb_exit(context);
    return 0;
}
